@@ -50,7 +50,8 @@ class TaxonomyProcessor:
         # Initialize data containers
         self.asvtable = None
         self.taxa_split = None
-        self.sampletable_randomID = None
+        self.sampletable_randomID = None  # For MALLET (uses randomID)
+        self.sampletable_genusID = None   # For Gensim (uses genus_ID)
 
     def _setup_directories(self):
         """Create necessary directories for analysis."""
@@ -159,20 +160,33 @@ class TaxonomyProcessor:
         # Save intermediate taxonomy file
         self.taxa_split.to_csv(os.path.join(self.intermediate_directory, 'intermediate_taxa.csv'), index=True)
 
-        # Create sample table with random IDs
-        mapping_dict = self.taxa_split['randomID'].to_dict()
-        self.sampletable_randomID = sampletable.copy()
-        new_columns = {}
+        # Create sample table with genus IDs for Gensim (readable names)
+        # This makes Gensim results interpretable and comparable
+        genus_mapping_dict = self.taxa_split['genus_ID'].to_dict()
+        self.sampletable_genusID = sampletable.copy()
+        new_columns_genus = {}
         for col in sampletable.columns:
-            if col in mapping_dict:
-                new_columns[col] = mapping_dict[col]
+            if col in genus_mapping_dict:
+                new_columns_genus[col] = genus_mapping_dict[col]
             else:
-                new_columns[col] = col
-        self.sampletable_randomID = self.sampletable_randomID.rename(columns=new_columns)
+                new_columns_genus[col] = col
+        self.sampletable_genusID = self.sampletable_genusID.rename(columns=new_columns_genus)
+        self.sampletable_genusID.to_csv(os.path.join(self.intermediate_directory, 'annotated_genusid.csv'), index=True)
+        
+        # Create sample table with random IDs for MALLET (backward compatibility)
+        random_mapping_dict = self.taxa_split['randomID'].to_dict()
+        self.sampletable_randomID = sampletable.copy()
+        new_columns_random = {}
+        for col in sampletable.columns:
+            if col in random_mapping_dict:
+                new_columns_random[col] = random_mapping_dict[col]
+            else:
+                new_columns_random[col] = col
+        self.sampletable_randomID = self.sampletable_randomID.rename(columns=new_columns_random)
         self.sampletable_randomID.to_csv(os.path.join(self.intermediate_directory, 'annotaed_randomid.csv'), index=True)
 
     def create_mallet_input(self):
-        """Create Mallet input documents from processed data."""
+        """Create MALLET input documents from processed data (uses randomID)."""
         if self.sampletable_randomID is None:
             raise ValueError("Data must be loaded and processed first. Call load_and_process_data().")
 
@@ -194,24 +208,28 @@ class TaxonomyProcessor:
     def process_all(self):
         """
         Run the complete processing pipeline.
+        Creates MALLET text file (randomID) and both CSV matrices.
+        Gensim loads directly from annotated_genusid.csv matrix.
 
         Returns:
             dict: Dictionary containing processed data and file paths
         """
         self.load_and_process_data()
-        self.create_mallet_input()
+        self.create_mallet_input()  # For MALLET (randomID) - creates training.txt
+        # Note: Gensim loads directly from annotated_genusid.csv, no text file needed
 
         return {
             'asvtable': self.asvtable,
             'taxa_split': self.taxa_split,
-            'sampletable_randomID': self.sampletable_randomID,
+            'sampletable_randomID': self.sampletable_randomID,  # For MALLET
+            'sampletable_genusID': self.sampletable_genusID,    # For Gensim
             'paths': {
                 'intermediate_directory': self.intermediate_directory,
                 'loop_directory': self.loop_directory,
                 'lda_directory': self.lda_directory,
                 'MC_sample_directory': self.MC_sample_directory,
                 'MC_feature_directory': self.MC_feature_directory,
-                'path_to_training_data': self.path_to_training_data,
+                'path_to_training_data': self.path_to_training_data,  # MALLET training.txt (randomID)
                 'path_to_formatted_training_data': self.path_to_formatted_training_data
             }
-        }
\ No newline at end of file
+        }
