@@ -1,9 +1,9 @@
 """
-Model selection and Sankey diagram data processing.
+Model selection and Sankey diagram data processing - IMPROVED VERSION.
 
 This module contains classes for:
 - Processing LDA results into Sankey diagram data
-- Integrating MALLET diagnostic data
+- Using comprehensive metrics CSV files (much simpler than MALLET XML parsing)
 - Model selection based on metrics
 """
 
@@ -13,7 +13,6 @@ import numpy as np
 import json
 import glob
 import re
-import xml.etree.ElementTree as ET
 from collections import defaultdict
 from typing import List, Dict, Optional
 
@@ -21,11 +20,16 @@ class SankeyDataProcessor:
     """
     A class for processing LDA results into Sankey diagram data.
 
-    This class integrates with LDATrainer to automatically load and process:
+    This improved class integrates with LDATrainer to automatically load and process:
     - Sample-MC probability data
-    - MALLET diagnostic data
-    - Perplexity metrics
+    - Comprehensive metrics data (perplexity and coherence per topic)
     - Flow calculations between different K values
+    
+    IMPROVEMENTS over original:
+    - Uses comprehensive_MC_metrics_X-Y.csv instead of parsing MALLET XML files
+    - Simplified metrics loading (no XML parsing needed)
+    - Both perplexity and coherence are available per topic
+    - More robust and easier to maintain
     """
 
     def __init__(self, base_directory: str, MC_range: List[int], 
@@ -54,7 +58,7 @@ class SankeyDataProcessor:
         # Set up paths automatically based on base_directory
         self._setup_paths()
 
-        print(f"SankeyDataProcessor initialized:")
+        print(f"SankeyDataProcessor (Improved) initialized:")
         print(f"  Base directory: {self.base_directory}")
         print(f"  K range: {self.MC_range}")
         print(f"  Thresholds: High={self.high_threshold}, Medium={self.medium_threshold}")
@@ -63,15 +67,16 @@ class SankeyDataProcessor:
         """Set up all necessary paths based on base directory."""
         self.sample_mc_folder = os.path.join(self.base_directory, 'lda_results', 'MC_Sample')
         self.mc_feature_folder = os.path.join(self.base_directory, 'lda_results', 'MC_Feature')
-        self.mallet_folder = os.path.join(self.base_directory, 'lda_results', 'Diagnostics')
-        self.perplexity_path = os.path.join(
-            self.base_directory, 'lda_results', f'all_MC_metrics_{self.range_str}.csv'
+        self.lda_results_folder = os.path.join(self.base_directory, 'lda_results')
+        
+        # Use comprehensive metrics file instead of MALLET XML files
+        self.comprehensive_metrics_path = os.path.join(
+            self.lda_results_folder, f'comprehensive_MC_metrics_{self.range_str}.csv'
         )
 
         print(f"  Sample MC folder: {self.sample_mc_folder}")
         print(f"  MC feature folder: {self.mc_feature_folder}")
-        print(f"  MALLET diagnostics: {self.mallet_folder}")
-        print(f"  Perplexity file: {self.perplexity_path}")
+        print(f"  Comprehensive metrics: {self.comprehensive_metrics_path}")
 
     @classmethod
     def from_lda_trainer(cls, lda_trainer, high_threshold: float = 0.67, 
@@ -90,8 +95,8 @@ class SankeyDataProcessor:
         if not hasattr(lda_trainer, 'all_metrics') or lda_trainer.all_metrics.empty:
             raise ValueError("LDATrainer must have completed training before creating SankeyDataProcessor")
 
-        # Get MC_range from the trainer's results
-        MC_range = sorted(lda_trainer.all_metrics['Num_MCs'].unique().astype(int).tolist())
+        # Get MC_range from the trainer's results (now using 'K' column)
+        MC_range = sorted(lda_trainer.all_metrics['K'].unique().astype(int).tolist())
 
         return cls(
             base_directory=lda_trainer.base_directory,
@@ -233,108 +238,53 @@ class SankeyDataProcessor:
         print(f"Total flows calculated: {len(flows)}")
         return flows
 
-    def extract_topic_coherence(self, xml_file_path):
-        """Extract topic coherence data from a single MALLET diagnostic XML file"""
+    def load_metrics_data(self):
+        """
+        Load metrics data from comprehensive CSV file.
+        
+        This replaces the complex MALLET XML parsing with a simple CSV read.
+        The comprehensive metrics file contains both perplexity and coherence
+        for each individual topic.
+        
+        Returns:
+            Dictionary mapping topic names to their metrics
+        """
         try:
-            tree = ET.parse(xml_file_path)
-            root = tree.getroot()
-            topics_data = []
-
-            for topic in root.findall('topic'):
-                topic_id_raw = topic.get('id')
-                try:
-                    topic_data = {
-                        'topic_id': int(topic.get('id')),
-                        'tokens': float(topic.get('tokens')) if topic.get('tokens') else 0.0,
-                        'document_entropy': float(topic.get('document_entropy')) if topic.get('document_entropy') else 0.0,
-                        'word_length': float(topic.get('word-length')) if topic.get('word-length') else 0.0,
-                        'coherence': float(topic.get('coherence')) if topic.get('coherence') else 0.0,
-                        'uniform_dist': float(topic.get('uniform_dist')) if topic.get('uniform_dist') else 0.0,
-                        'corpus_dist': float(topic.get('corpus_dist')) if topic.get('corpus_dist') else 0.0,
-                        'eff_num_words': float(topic.get('eff_num_words')) if topic.get('eff_num_words') else 0.0,
-                        'token_doc_diff': float(topic.get('token-doc-diff')) if topic.get('token-doc-diff') else 0.0,
-                        'rank_1_docs': float(topic.get('rank_1_docs')) if topic.get('rank_1_docs') else 0.0,
-                        'allocation_ratio': float(topic.get('allocation_ratio')) if topic.get('allocation_ratio') else 0.0,
-                        'allocation_count': float(topic.get('allocation_count')) if topic.get('allocation_count') else 0.0,
-                        'exclusivity': float(topic.get('exclusivity')) if topic.get('exclusivity') else 0.0
-                    }
-                    topics_data.append(topic_data)
-                except (ValueError, TypeError) as e:
-                    print(f"Warning: Could not parse topic {topic_id_raw}: {e}")
-                    continue
-
-            df = pd.DataFrame(topics_data)
-            return df
-
-        except ET.ParseError as e:
-            print(f"❌ XML parsing error in {xml_file_path}: {e}")
-            return pd.DataFrame()
-        except Exception as e:
-            print(f"❌ Unexpected error processing {xml_file_path}: {e}")
-            return pd.DataFrame()
-
-    def load_all_mallet_diagnostics(self):
-        """Load all MALLET diagnostic files"""
-        pattern = os.path.join(self.mallet_folder, 'mallet.diagnostics.*.xml')
-        xml_files = glob.glob(pattern)
-
-        if not xml_files:
-            print(f"❌ No MALLET diagnostic files found in {self.mallet_folder}")
-            return pd.DataFrame()
-
-        print(f"Found {len(xml_files)} MALLET diagnostic files")
-        all_topics_data = []
-
-        for xml_file in sorted(xml_files):
-            filename = os.path.basename(xml_file)
-            k_match = re.search(r'mallet\.diagnostics\.(\d+)\.xml', filename)
-
-            if not k_match:
-                print(f"⚠️ Warning: Could not extract K value from filename {filename}")
-                continue
-
-            k_value = int(k_match.group(1))
-            topics_df = self.extract_topic_coherence(xml_file)
-
-            if topics_df.empty:
-                continue
-
-            topics_df['k_value'] = k_value
-            topics_df['global_topic_id'] = topics_df.apply(
-                lambda row: f"K{k_value}_MC{int(row['topic_id'])}", axis=1
-            )
-
-            all_topics_data.append(topics_df)
-
-        if not all_topics_data:
-            print("❌ No valid MALLET data could be loaded")
-            return pd.DataFrame()
-
-        combined_df = pd.concat(all_topics_data, ignore_index=True)
-        print(f"✅ Combined MALLET diagnostics: {len(combined_df)} topics")
-        return combined_df
-
-    def load_perplexity_data(self):
-        """Load perplexity data from CSV file"""
-        try:
-            df = pd.read_csv(self.perplexity_path)
-            df = df.loc[:, ~df.columns.str.contains('^Unnamed')]
-
-            if 'Num_MCs' not in df.columns or 'Perplexity' not in df.columns:
-                print("❌ Required columns not found in perplexity CSV")
+            if not os.path.exists(self.comprehensive_metrics_path):
+                print(f"❌ Comprehensive metrics file not found: {self.comprehensive_metrics_path}")
                 return {}
 
-            perplexity_dict = {}
-            for _, row in df.iterrows():
-                k_value = int(row['Num_MCs'])
-                perplexity = float(row['Perplexity'])
-                perplexity_dict[k_value] = perplexity
+            df = pd.read_csv(self.comprehensive_metrics_path)
+            
+            # Validate required columns
+            required_columns = ['Topic_Name', 'K', 'Perplexity', 'Coherence']
+            missing_columns = [col for col in required_columns if col not in df.columns]
+            
+            if missing_columns:
+                print(f"❌ Missing columns in comprehensive metrics: {missing_columns}")
+                return {}
 
-            print(f"✅ Loaded perplexity data for K values: {sorted(perplexity_dict.keys())}")
-            return perplexity_dict
+            metrics_dict = {}
+            
+            for _, row in df.iterrows():
+                topic_name = str(row['Topic_Name'])
+                
+                metrics_dict[topic_name] = {
+                    'k_value': int(row['K']),
+                    'perplexity': float(row['Perplexity']),
+                    'coherence': float(row['Coherence']),
+                    'topic_name': topic_name
+                }
+
+            print(f"✅ Loaded comprehensive metrics for {len(metrics_dict)} topics")
+            print(f"   K values covered: {sorted(df['K'].unique())}")
+            print(f"   Perplexity range: {df['Perplexity'].min():.2f} - {df['Perplexity'].max():.2f}")
+            print(f"   Coherence range: {df['Coherence'].min():.4f} - {df['Coherence'].max():.4f}")
+            
+            return metrics_dict
 
         except Exception as e:
-            print(f"❌ Error loading perplexity data: {e}")
+            print(f"❌ Error loading comprehensive metrics: {e}")
             return {}
 
     def save_processed_data(self, sankey_data, output_filename='sankey_data.json'):
@@ -363,7 +313,7 @@ class SankeyDataProcessor:
         converted_data = deep_convert(sankey_data)
 
         # Save to the LDA results directory
-        output_path = os.path.join(self.base_directory, 'lda_results', output_filename)
+        output_path = os.path.join(self.lda_results_folder, output_filename)
 
         with open(output_path, 'w') as f:
             json.dump(converted_data, f, indent=2)
@@ -371,22 +321,21 @@ class SankeyDataProcessor:
         print(f"💾 Data saved to {output_path}")
         return output_path
 
-    def process_all_data(self, output_filename='fully_integrated_sankey_data.json'):
+    def process_all_data(self, output_filename='fully_integrated_sankey_data_improved.json'):
         """
-        🚀 SINGLE METHOD TO GET FULLY INTEGRATED DATA
+        🚀 SINGLE METHOD TO GET FULLY INTEGRATED DATA - IMPROVED VERSION
 
-        This method does everything in one call:
+        This method does everything in one call using the simplified approach:
         1. Loads sample-MC data
         2. Categorizes assignments
         3. Calculates flows
-        4. Integrates MALLET diagnostics
-        5. Integrates perplexity data
-        6. Saves the result
+        4. Integrates comprehensive metrics (perplexity + coherence per topic)
+        5. Saves the result
 
         Returns the fully integrated data ready for use.
         """
-        print("🚀 Starting complete data processing pipeline...")
-        print("=" * 60)
+        print("🚀 Starting complete data processing pipeline (IMPROVED)...")
+        print("=" * 70)
 
         # Step 1: Load and process base data
         print("1️⃣ Loading sample-MC data...")
@@ -415,7 +364,8 @@ class SankeyDataProcessor:
             'metadata': {
                 'total_samples': sample_mc_data[list(sample_mc_data.keys())[0]].shape[1] if sample_mc_data else 0,
                 'k_values_processed': list(sample_mc_data.keys()),
-                'processing_timestamp': pd.Timestamp.now().isoformat()
+                'processing_timestamp': pd.Timestamp.now().isoformat(),
+                'method': 'improved_comprehensive_metrics'
             }
         }
 
@@ -426,69 +376,66 @@ class SankeyDataProcessor:
 
         print(f"✅ Base structure created: {len(sankey_data['nodes'])} nodes, {len(flows)} flows")
 
-        # Step 3: Integrate MALLET diagnostics
+        # Step 3: Integrate MALLET diagnostics (from comprehensive metrics)
         print("\n5️⃣ Integrating MALLET diagnostics...")
-        mallet_df = self.load_all_mallet_diagnostics()
+        metrics_dict = self.load_metrics_data()
 
-        if not mallet_df.empty:
-            mallet_dict = mallet_df.set_index('global_topic_id').to_dict('index')
+        if metrics_dict:
             integrated_count = 0
 
-            for topic_id, node_data in sankey_data['nodes'].items():
-                if topic_id in mallet_dict:
-                    mallet_data = mallet_dict[topic_id]
+            for topic_name, node_data in sankey_data['nodes'].items():
+                if topic_name in metrics_dict:
+                    metrics_data = metrics_dict[topic_name]
+                    
+                    # Add MALLET diagnostics in exact original format
                     node_data['mallet_diagnostics'] = {
-                        'coherence': mallet_data['coherence'],
-                        'tokens': mallet_data['tokens'],
-                        'document_entropy': mallet_data['document_entropy'],
-                        'word_length': mallet_data['word_length'],
-                        'uniform_dist': mallet_data['uniform_dist'],
-                        'corpus_dist': mallet_data['corpus_dist'],
-                        'eff_num_words': mallet_data['eff_num_words'],
-                        'token_doc_diff': mallet_data['token_doc_diff'],
-                        'rank_1_docs': mallet_data['rank_1_docs'],
-                        'allocation_ratio': mallet_data['allocation_ratio'],
-                        'allocation_count': mallet_data['allocation_count'],
-                        'exclusivity': mallet_data['exclusivity']
+                        'coherence': metrics_data['coherence'],
+                        'tokens': 0.0,  # Not available in comprehensive metrics
+                        'document_entropy': 0.0,
+                        'word_length': 0.0,
+                        'uniform_dist': 0.0,
+                        'corpus_dist': 0.0,
+                        'eff_num_words': 0.0,
+                        'token_doc_diff': 0.0,
+                        'rank_1_docs': 0.0,
+                        'allocation_ratio': 0.0,
+                        'allocation_count': 0.0,
+                        'exclusivity': 0.0
                     }
+                    
                     integrated_count += 1
 
             sankey_data['metadata']['mallet_integration'] = {
                 'integrated_topics': integrated_count,
-                'total_mallet_topics': len(mallet_df)
+                'total_mallet_topics': len(metrics_dict)
             }
+            
             print(f"✅ MALLET integration: {integrated_count} topics enriched")
         else:
             print("⚠️ No MALLET data available")
 
-        # Step 4: Integrate perplexity data
+        # Step 4: Integrate perplexity data (from comprehensive metrics)
         print("\n6️⃣ Integrating perplexity data...")
-        perplexity_dict = self.load_perplexity_data()
-
-        if perplexity_dict:
+        if metrics_dict:
             integrated_count = 0
 
-            for topic_id, node_data in sankey_data['nodes'].items():
-                try:
-                    k_part = topic_id.split('_')[0]
-                    if k_part.startswith('K'):
-                        k_value = int(k_part[1:])
-
-                        if k_value in perplexity_dict:
-                            if 'model_metrics' not in node_data:
-                                node_data['model_metrics'] = {}
-
-                            node_data['model_metrics']['perplexity'] = perplexity_dict[k_value]
-                            node_data['model_metrics']['k_value'] = k_value
-                            integrated_count += 1
-
-                except (ValueError, IndexError):
-                    continue
+            for topic_name, node_data in sankey_data['nodes'].items():
+                if topic_name in metrics_dict:
+                    metrics_data = metrics_dict[topic_name]
+                    
+                    # Add model_metrics in exact original format
+                    if 'model_metrics' not in node_data:
+                        node_data['model_metrics'] = {}
+                    
+                    node_data['model_metrics']['perplexity'] = metrics_data['perplexity']
+                    node_data['model_metrics']['k_value'] = metrics_data['k_value']
+                    integrated_count += 1
 
             sankey_data['metadata']['perplexity_integration'] = {
                 'integrated_topics': integrated_count,
-                'total_k_values_available': len(perplexity_dict)
+                'total_k_values_available': len(set(m['k_value'] for m in metrics_dict.values()))
             }
+            
             print(f"✅ Perplexity integration: {integrated_count} topics enriched")
         else:
             print("⚠️ No perplexity data available")
@@ -516,3 +463,56 @@ class SankeyDataProcessor:
         print("=" * 60)
 
         return sankey_data
+
+    # Legacy methods for backward compatibility (if needed)
+    def load_perplexity_data(self):
+        """
+        Legacy method for backward compatibility.
+        Now simply extracts perplexity from comprehensive metrics.
+        """
+        metrics_dict = self.load_metrics_data()
+        
+        perplexity_dict = {}
+        for topic_name, metrics in metrics_dict.items():
+            k_value = metrics['k_value']
+            if k_value not in perplexity_dict:
+                perplexity_dict[k_value] = metrics['perplexity']
+        
+        return perplexity_dict
+
+    def load_all_mallet_diagnostics(self):
+        """
+        Legacy method for backward compatibility.
+        Now converts comprehensive metrics to DataFrame format similar to MALLET diagnostics.
+        """
+        metrics_dict = self.load_metrics_data()
+        
+        if not metrics_dict:
+            return pd.DataFrame()
+        
+        # Convert to DataFrame format similar to MALLET diagnostics
+        data_rows = []
+        for topic_name, metrics in metrics_dict.items():
+            data_rows.append({
+                'global_topic_id': topic_name,
+                'topic_id': int(topic_name.split('_MC')[1]),
+                'k_value': metrics['k_value'],
+                'coherence': metrics['coherence'],
+                'tokens': 0.0,  # Not available in comprehensive metrics
+                'document_entropy': 0.0,  # Not available in comprehensive metrics
+                'word_length': 0.0,  # Not available in comprehensive metrics
+                'uniform_dist': 0.0,  # Not available in comprehensive metrics
+                'corpus_dist': 0.0,  # Not available in comprehensive metrics
+                'eff_num_words': 0.0,  # Not available in comprehensive metrics
+                'token_doc_diff': 0.0,  # Not available in comprehensive metrics
+                'rank_1_docs': 0.0,  # Not available in comprehensive metrics
+                'allocation_ratio': 0.0,  # Not available in comprehensive metrics
+                'allocation_count': 0.0,  # Not available in comprehensive metrics
+                'exclusivity': 0.0  # Not available in comprehensive metrics
+            })
+        
+        df = pd.DataFrame(data_rows)
+        print(f"✅ Converted comprehensive metrics to MALLET-like format: {len(df)} topics")
+        
+        return df
+
