@@ -1,7 +1,7 @@
 import anywidget
 import traitlets
 
-class StripeSankeyInline_test(anywidget.AnyWidget):
+class StripeSankeyInline(anywidget.AnyWidget):
     _esm = """
     import * as d3 from "https://cdn.skypack.dev/d3@7";
 
