@@ -75,9 +75,7 @@ class LDATrainer:
             'iterations': 400,
             'alpha': 'auto',
             'eta': 'auto',
-            'random_state': 42,
-            'chunksize': 2000,
-            'eval_every': None
+            'random_state': 42
         }
         # Update with user-provided parameters
         self.gensim_params.update(gensim_params)
@@ -136,39 +134,66 @@ class LDATrainer:
 
     def load_training_data(self):
         """
-        Load training data from files created by TaxonomyProcessor.
+        Load training data from intermediate files.
 
-        This method automatically loads:
-        - flattened_nested_list from training.txt
-        - sampletable_genusid from annotated_randomid.csv
-        """
-        # Load the sample table with random IDs
-        sampletable_path = os.path.join(self.paths['intermediate_directory'], 'annotaed_randomid.csv')
-        if not os.path.exists(sampletable_path):
-            raise FileNotFoundError(f"Sample table not found: {sampletable_path}. Run TaxonomyProcessor first.")
-
-        self.sampletable_genusid = pd.read_csv(sampletable_path, index_col=0)
-        print(f"  ✓ Loaded sample table: {self.sampletable_genusid.shape}")
-
-        # Load the flattened nested list from training data
-        training_data_path = self.paths['path_to_training_data']
-        if not os.path.exists(training_data_path):
-            raise FileNotFoundError(f"Training data not found: {training_data_path}. Run TaxonomyProcessor first.")
-
-        with open(training_data_path, 'r') as f:
-            self.flattened_nested_list = [line.strip() for line in f]
-
-        print(f"  ✓ Loaded training documents: {len(self.flattened_nested_list)} documents")
+        Input: Reads from intermediate directory:
+               - For Gensim: annotated_genusid.csv (matrix with genus IDs)
+               - For MALLET: training.txt (text file with randomIDs) and annotaed_randomid.csv
         
-        # Prepare Gensim data if using Gensim
+        Output: Sets instance attributes:
+                - self.flattened_nested_list: List of document strings (MALLET only)
+                - self.sampletable_genusid: DataFrame with sample abundances
+                - self.gensim_corpus: Bag-of-words corpus (if using Gensim)
+                - self.gensim_dictionary: Gensim Dictionary (if using Gensim)
+                - self.processed_texts: List of tokenized documents (if using Gensim)
+        """
         if self.implementation == 'gensim':
-            self._prepare_gensim_data()
+            # Gensim: Load matrix directly with genus IDs (readable names)
+            sampletable_path = os.path.join(self.paths['intermediate_directory'], 'annotated_genusid.csv')
+            if not os.path.exists(sampletable_path):
+                raise FileNotFoundError(f"Sample table not found: {sampletable_path}. Run TaxonomyProcessor first.")
+            
+            self.sampletable_genusid = pd.read_csv(sampletable_path, index_col=0)
+            print(f"Loading Gensim data directly from matrix (genus IDs)...")
+            print(f"  ✓ Loaded sample table: {self.sampletable_genusid.shape}")
+            
+            # Prepare Gensim data directly from matrix
+            self._prepare_gensim_data_from_matrix()
+            
+        else:
+            # MALLET: Load text file with random IDs (MALLET requires text input)
+            sampletable_path = os.path.join(self.paths['intermediate_directory'], 'annotaed_randomid.csv')
+            if not os.path.exists(sampletable_path):
+                raise FileNotFoundError(f"Sample table not found: {sampletable_path}. Run TaxonomyProcessor first.")
+
+            self.sampletable_genusid = pd.read_csv(sampletable_path, index_col=0)
+            print(f"Loading MALLET data from text file (random IDs)...")
+            print(f"  ✓ Loaded sample table: {self.sampletable_genusid.shape}")
+
+            # Load the flattened nested list from training data
+            training_data_path = self.paths['path_to_training_data']
+            if not os.path.exists(training_data_path):
+                raise FileNotFoundError(f"Training data not found: {training_data_path}. Run TaxonomyProcessor first.")
+
+            with open(training_data_path, 'r') as f:
+                self.flattened_nested_list = [line.strip() for line in f]
+
+            print(f"  ✓ Loaded training documents: {len(self.flattened_nested_list)} documents")
         
         print("Training data loaded successfully.")
 
     def _prepare_gensim_data(self):
-        """Prepare data for Gensim training."""
-        print("Preparing data for Gensim...")
+        """
+        Prepare data for Gensim training from text file (legacy method).
+        
+        Input: self.flattened_nested_list - List of document strings (space-separated tokens)
+        
+        Output: Sets instance attributes:
+                - self.processed_texts: List of lists of tokens
+                - self.gensim_dictionary: Gensim Dictionary mapping tokens to IDs
+                - self.gensim_corpus: List of bag-of-words documents [(token_id, count), ...]
+        """
+        print("Preparing data for Gensim from text file...")
         
         # Convert documents to list of word lists
         self.processed_texts = []
@@ -182,6 +207,41 @@ class LDATrainer:
         
         print(f"  ✓ Created dictionary with {len(self.gensim_dictionary)} unique terms")
         print(f"  ✓ Created corpus with {len(self.gensim_corpus)} documents")
+    
+    def _prepare_gensim_data_from_matrix(self):
+        """
+        Prepare data for Gensim training directly from abundance matrix.
+        More efficient than text file - no intermediate file needed.
+        
+        Input: self.sampletable_genusid - DataFrame with samples as rows, genus_IDs as columns
+        
+        Output: Sets instance attributes:
+                - self.processed_texts: List of lists of tokens (for coherence calculation)
+                - self.gensim_dictionary: Gensim Dictionary mapping genus_IDs to IDs
+                - self.gensim_corpus: List of bag-of-words documents [(token_id, count), ...]
+        """
+        print("Preparing data for Gensim directly from matrix...")
+        
+        # Create processed_texts by repeating genus names by their counts
+        self.processed_texts = []
+        for idx, row in self.sampletable_genusid.iterrows():
+            doc_tokens = []
+            for genus_id, count in row.items():
+                if count > 0:
+                    # Repeat genus_id by its count
+                    doc_tokens.extend([str(genus_id)] * int(count))
+            self.processed_texts.append(doc_tokens)
+        
+        # Create Gensim dictionary from processed texts (not just column names!)
+        # This ensures the dictionary sees the words in their document context
+        self.gensim_dictionary = Dictionary(self.processed_texts)
+        
+        # Create corpus using the dictionary
+        self.gensim_corpus = [self.gensim_dictionary.doc2bow(text) for text in self.processed_texts]
+        
+        print(f"  ✓ Created dictionary with {len(self.gensim_dictionary)} unique terms")
+        print(f"  ✓ Created corpus with {len(self.gensim_corpus)} documents")
+        print(f"  ✓ Using genus IDs (readable names) for interpretable results")
 
     def _generate_file_paths(self, num_topics: int) -> Dict[str, str]:
         """Generate all file paths for a specific number of topics."""
@@ -303,24 +363,33 @@ class LDATrainer:
             topic_probs = [doc_topics.get(topic_id, 0.0) for topic_id in range(num_topics)]
             topic_distributions.append(topic_probs)
         
-        # Get word-topic distributions
-        word_topics = []
-        for topic_id in range(num_topics):
-            topic_words = lda_model.show_topic(topic_id, topn=len(self.gensim_dictionary))
-            for word, prob in topic_words:
-                # Convert probability to frequency-like score (multiply by large number for compatibility)
-                freq = int(prob * 10000)
-                if freq > 0:  # Only include words with non-zero probability
-                    word_topics.append((topic_id, word, freq))
+        # Get word-topic distributions using get_topics() for full precision
+        # This returns a (num_topics x vocab_size) matrix with full floating-point probabilities
+        topic_word_matrix = lda_model.get_topics()  # Shape: (num_topics, vocab_size)
         
         # Create topic index
         topic_index = self._create_topic_index(num_topics)
         
-        # Process ASV data
-        df_asv = pd.DataFrame(word_topics, columns=['MC', 'Term', 'Frequency'])
-        df_asv_pivot = df_asv.pivot_table(index='MC', columns='Term', values='Frequency', fill_value=0)
-        df_asv_probabilities = df_asv_pivot.div(df_asv_pivot.sum(axis=1), axis=0)
-        df_asv_probabilities.index = topic_index
+        # Get vocabulary in the correct order
+        vocab = [self.gensim_dictionary[i] for i in range(len(self.gensim_dictionary))]
+        
+        # Create feature probabilities DataFrame directly from the matrix
+        # This preserves full precision and includes all words
+        df_asv_probabilities = pd.DataFrame(
+            topic_word_matrix,
+            index=topic_index,
+            columns=vocab
+        )
+        
+        # For backward compatibility, also create word_topics list
+        # Convert matrix back to list of tuples for any downstream code that expects it
+        word_topics = []
+        for topic_id in range(num_topics):
+            for word_id, prob in enumerate(topic_word_matrix[topic_id]):
+                if prob > 0:  # Only include non-zero probabilities
+                    word = self.gensim_dictionary[word_id]
+                    # Store as (topic_id, word, probability) - keeping full precision
+                    word_topics.append((topic_id, word, prob))
         
         # Create topic distribution DataFrame
         df_topic_dist = pd.DataFrame(
